unit DataSetList;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  db, dbtables;

type
  TDataSetEntry = class(TCollectionItem)
  private
    FSelected : Boolean;
    FTable : TDataSet;
    FBookmark : TBookmark;
  protected
    function GetSelected : Boolean;
    procedure SetSelected(what: Boolean);
    function GetTable : TDataSet;
    procedure SetTable(tbl: TDataSet);
  public
    constructor Create(Collection: TCollection); override;
    function GetBookmark : TBookmark;
    procedure GotoBookMark;
    procedure FreeBookMark;
  published
    property Selected: Boolean read GetSelected write SetSelected;
    property Table: TDataSet read GetTable write SetTable;
  end;

  TDataSetEntryProc=procedure(var T : TDataSetEntry);

  TDataSetEntries = class(TCollection)
  private
      FOwner: TComponent;
   protected
      function GetOwner: TPersistent; override;
   public
      function Add: TDataSetEntry;
      function Insert( Index: Integer): TDataSetEntry;
      function FindItemID( ID: Integer): TDataSetEntry;
      function GetItem( Index: Integer): TDataSetEntry;
      procedure SetItem( Index: Integer; src: TDataSetEntry);
      constructor Create( Owner: TComponent);
      destructor Destroy; override;
      property Items[ Index: Integer]: TDataSetEntry read GetItem write SetItem;
 end;

 TOnIterateProc=procedure(Sender : TDataSetEntry;var Filter:Boolean) of Object;
 
  TDataSetList = class(TComponent)
  private
    { Private Declarations }
    FItems : TDataSetEntries;
    FUseSubset: Boolean;
    FIterateBeforeClearFields,
    FIterateAfterClearFields,
    FIterateBeforeClose,
    FIterateAfterClose,
    FIterateBeforeDisableControls,
    FIterateAfterDisableControls,
    FIterateBeforeEdit,
    FIterateAfterEdit,
    FIterateBeforeEnableControls,
    FIterateAfterEnableControls ,
    FIterateBeforeFirst,
    FIterateAfterFirst,
    FIterateBeforeLast,
    FIterateAfterLast,
    FIterateBeforeMoveBy,
    FIterateAfterMoveBy,
    FIterateBeforeNext,
    FIterateAfterNext,
    FIterateBeforeOpen,
    FIterateAfterOpen,
    FIterateBeforePost,
    FIterateAfterPost,
    FIterateBeforePrior,
    FIterateAfterPrior,
    FIterateBeforeRefresh,
    FIterateAfterRefresh,
    FIterateBeforeSetBookMark,
    FIterateAfterSetBookmark,
    FIterateBeforeReleaseBookMark,
    FIterateAfterReleaseBookMark
    : TOnIterateProc;
  protected
    { Protected Declarations }
    procedure Loaded; override;
  public
    { Public Declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ClearFields;
    procedure Close;
    procedure DisableControls;
    procedure Edit;
    procedure EnableControls;
    procedure First;
    procedure Last;
    procedure MoveBy(Distance: Integer);
    procedure Next;
    procedure Open;
    procedure Post;
    procedure Prior;
    procedure Refresh;
    procedure SetBookMark;
    procedure ReleaseBookMark;
  published
    { Published Declarations }
    property Items : TDataSetEntries read FItems write FItems;
    property UseSubset : Boolean read FUseSubset write FUseSubset;
    property OnBeforeClearFieldsIterate: TOnIterateProc read FIterateBeforeClearFields write FIterateBeforeClearFields;
    property OnAfterClearFieldsIterate: TOnIterateProc read FIterateAfterClearFields write FIterateAfterClearFields;
    property OnBeforeCloseIterate: TOnIterateProc read FIterateBeforeClose write FIterateBeforeClose;
    property OnAfterCloseIterate: TOnIterateProc read FIterateAfterClose write FIterateAfterClose;
    property OnBeforeDisableControlsIterate: TOnIterateProc read FIterateBeforeDisableControls write FIterateBeforeDisableControls;
    property OnAfterDisableControlsIterate: TOnIterateProc read FIterateAfterDisableControls write FIterateAfterDisableControls;
    property OnBeforeEditIterate: TOnIterateProc read FIterateBeforeEdit write FIterateBeforeEdit;
    property OnAfterEditIterate: TOnIterateProc read FIterateAfterEdit write FIterateAfterEdit;
    property OnBeforeEnableControlsIterate: TOnIterateProc read FIterateBeforeEnableControls write FIterateBeforeEnableControls;
    property OnAfterEnableControlsIterate: TOnIterateProc read FIterateAfterEnableControls write FIterateAfterEnableControls;
    property OnBeforeFirstIterate: TOnIterateProc read FIterateBeforeFirst write FIterateBeforeFirst;
    property OnAfterFirstIterate: TOnIterateProc read FIterateAfterFirst write FIterateAfterFirst;
    property OnBeforeLastIterate: TOnIterateProc read FIterateBeforeLast write FIterateBeforeLast;
    property OnAfterLastIterate: TOnIterateProc read FIterateAfterLast write FIterateAfterLast;
    property OnBeforeMoveByIterate: TOnIterateProc read FIterateBeforeMoveBy write FIterateBeforeMoveBy;
    property OnAfterMoveByIterate: TOnIterateProc read FIterateAfterMoveBy write FIterateAfterMoveBy;
    property OnBeforeNextIterate: TOnIterateProc read FIterateBeforeNext write FIterateBeforeNext;
    property OnAfterNextIterate: TOnIterateProc read FIterateAfterNext write FIterateAfterNext;
    property OnBeforeOpenIterate: TOnIterateProc read FIterateBeforeOpen write FIterateBeforeOpen;
    property OnAfterOpenIterate: TOnIterateProc read FIterateAfterOpen write FIterateAfterOpen;
    property OnBeforePostIterate: TOnIterateProc read FIterateBeforePost write FIterateBeforePost;
    property OnAfterPostIterate: TOnIterateProc read FIterateAfterPost write FIterateAfterPost;
    property OnBeforePriorIterate: TOnIterateProc read FIterateBeforePrior write FIterateBeforePrior;
    property OnAfterPriorIterate: TOnIterateProc read FIterateAfterPrior write FIterateAfterPrior;
    property OnBeforeRefreshIterate: TOnIterateProc read FIterateBeforeRefresh write FIterateBeforeRefresh;
    property OnAfterRefreshIterate: TOnIterateProc read FIterateAfterRefresh write FIterateAfterRefresh;
    property OnBeforeSetBookMarkIterate: TOnIterateProc read FIterateBeforeSetBookMark write FIterateBeforeSetBookMark;
    property OnAfterSetBookMarkIterate: TOnIterateProc read FIterateAfterSetBookMark write FIterateAfterSetBookMark;
    property OnBeforeReleaseBookMarkIterate: TOnIterateProc read FIterateBeforeReleaseBookMark write FIterateBeforeReleaseBookMark;
    property OnAfterReleaseBookMarkIterate: TOnIterateProc read FIterateAfterReleaseBookMark write FIterateAfterReleaseBookMark;
  end;

procedure Register;

implementation

{ TRegEntries }

function TDataSetEntries.Add: TDataSetEntry;
begin
   Result := TDataSetEntry( inherited Add);
end;

constructor TDataSetEntries.Create(Owner: TComponent);
begin
   inherited Create(TDataSetEntry);
   FOwner := Owner;
end;

destructor TDataSetEntries.Destroy;
begin
   FOwner := nil;
   inherited Destroy;
end;

function TDataSetEntries.FindItemID(ID: Integer): TDataSetEntry;
begin
   Result := TDataSetEntry( inherited FindItemID( ID));
end;

function TDataSetEntries.GetItem(Index: Integer): TDataSetEntry;
begin
   Result := TDataSetEntry( inherited GetItem( Index));
end;

function TDataSetEntries.GetOwner: TPersistent;
begin
   Result := FOwner;
end;

function TDataSetEntries.Insert(Index: Integer): TDataSetEntry;
begin
   Result := TDataSetEntry( inherited Insert( Index));
end;

procedure TDataSetEntries.SetItem(Index: Integer; src: TDataSetEntry);
begin
   inherited SetItem( Index, src);
end;

constructor TDataSetEntry.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FSelected := True;
end;

function TDataSetEntry.GetSelected : Boolean;
begin
  Result := FSelected;
end;

procedure TDataSetEntry.SetSelected(what: Boolean);
begin
  FSelected := what;
end;

function TDataSetEntry.GetTable : TDataSet;
begin
  Result := FTable;
end;

procedure TDataSetEntry.SetTable(tbl: TDataSet);
begin
  FTable := tbl;
end;

function TDataSetEntry.GetBookmark;
begin
   FBookmark := FTable.GetBookmark;
   Result := FBookmark;
end;

procedure TDataSetEntry.GotoBookMark;
begin
   FTable.GotoBookmark(FBookmark);
end;

procedure TDataSetEntry.FreeBookMark;
begin
   FTable.FreeBookmark(FBookMark);
end;

procedure TDataSetList.Loaded;
begin
  inherited Loaded;
  {component specific initialization code, etc.}
end;{Loaded}

constructor TDataSetList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  {Add any other initialization code here}
  FItems := TDataSetEntries.Create(Self);
  FUseSubset := true;
end;{Create}

destructor TDataSetList.Destroy;
begin
  FItems.Free;
  {Add any cleanup code here}
  inherited Destroy;
end;{Destroy}

procedure TDataSetList.ClearFields;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeClearFields) then
           FIterateBeforeClearFields(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.ClearFields;
        if Assigned(FIterateAfterClearfields) then
           FIterateAfterClearFields(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.Close;
var i : Integer;
    u : Boolean;
begin
   for i := FItems.Count - 1 downto 0 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeClose) then
           FIterateBeforeClose(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.Close;
        if Assigned(FIterateAfterClose) then
           FIterateAfterClose(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.DisableControls;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeDisableControls) then
           FIterateBeforeDisableControls(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.DisableControls;
        if Assigned(FIterateAfterDisableControls) then
           FIterateAfterDisableControls(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.EnableControls;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeEnableControls) then
           FIterateBeforeEnableControls(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.EnableControls;
        if Assigned(FIterateAfterEnableControls) then
           FIterateAfterEnableControls(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.Edit;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeEdit) then
           FIterateBeforeEdit(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.Edit;
        if Assigned(FIterateAfterEdit) then
           FIterateAfterEdit(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.First;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeFirst) then
           FIterateBeforeFirst(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.First;
        if Assigned(FIterateAfterFirst) then
           FIterateAfterFirst(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.Last;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeLast) then
           FIterateBeforeLast(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.Last;
        if Assigned(FIterateAfterLast) then
           FIterateAfterLast(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.MoveBy(Distance:Integer);
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeMoveBy) then
           FIterateBeforeMoveBy(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.MoveBy(Distance);
        if Assigned(FIterateAfterMoveBy) then
           FIterateAfterMoveBy(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.Next;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeNext) then
           FIterateBeforeNext(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.Next;
        if Assigned(FIterateAfterNext) then
           FIterateAfterNext(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.Open;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeOpen) then
           FIterateBeforeOpen(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.Open;
        if Assigned(FIterateAfterOpen) then
           FIterateAfterOpen(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.Post;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforePost) then
           FIterateBeforePost(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.Post;
        if Assigned(FIterateAfterPost) then
           FIterateAfterPost(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.Prior;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforePrior) then
           FIterateBeforePrior(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.Prior;
        if Assigned(FIterateAfterPrior) then
           FIterateAfterPrior(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.Refresh;
var i : Integer;
    u : Boolean;
begin
   for i := FItems.Count - 1 downto 0 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeRefresh) then
           FIterateBeforeRefresh(FItems.Items[i],u);
        if u then
           FItems.Items[i].FTable.Refresh;
        if Assigned(FIterateAfterRefresh) then
           FIterateAfterRefresh(FItems.Items[i],u);
     end;
   end;
end;

procedure TDataSetList.SetBookMark;
var i : Integer;
    u : Boolean;
begin
  for i := FItems.Count - 1 downto 0 do
  begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeSetBookMark) then
           FIterateBeforeSetBookMark(FItems.Items[i],u);
        if u then
            FItems.Items[i].GetBookmark;
        if Assigned(FIterateAfterSetBookMark) then
           FIterateAfterSetBookMark(FItems.Items[i],u);
     end;
  end;
end;

procedure TDataSetList.ReleaseBookMark;
var i : Integer;
    u : Boolean;
begin
   for i := 0 to FItems.Count - 1 do
   begin
     if ((not FUseSubset and not FItems.Items[i].FSelected) or FItems.Items[i].FSelected) then
     begin
        u := FItems.Items[i].FSelected;
        if Assigned(FIterateBeforeReleaseBookMark) then
           FIterateBeforeReleaseBookMark(FItems.Items[i],u);
        if u then
        begin
           FItems.Items[i].GotoBookmark;
           FItems.Items[i].FreeBookmark;
        end;
        if Assigned(FIterateAfterReleaseBookMark) then
           FIterateAfterReleaseBookMark(FItems.Items[i],u);
     end;
   end;
end;

procedure Register;
begin
  {Register component and add to Samples page in VCL}
  RegisterComponents('Samples', [TDataSetList]);
end;{Register}


end.
